package sites

import (
	"context"
	"errors"
	"lmika.dev/lmika/hugo-crm/config"
	"lmika.dev/lmika/hugo-crm/models"
	"lmika.dev/lmika/hugo-crm/providers/db"
	"lmika.dev/lmika/hugo-crm/providers/themes"
	"lmika.dev/lmika/hugo-crm/services/jobs"
	"lmika.dev/lmika/hugo-crm/services/sitebuilder"
	"strings"
	"unicode"
)

type Service struct {
	cfg    config.Config
	db     *db.DB
	themes *themes.Provider
	sb     *sitebuilder.Service
	jobs   *jobs.Service
}

func NewService(
	cfg config.Config,
	db *db.DB,
	themes *themes.Provider,
	sb *sitebuilder.Service,
	jobs *jobs.Service,
) *Service {
	return &Service{
		cfg:    cfg,
		db:     db,
		themes: themes,
		sb:     sb,
		jobs:   jobs,
	}
}

func (s *Service) GetSite(ctx context.Context, id int) (models.Site, error) {
	return s.db.GetSite(ctx, int64(id))
}

func (s *Service) CreateSite(ctx context.Context, name string) (models.Site, error) {
	newSite := models.Site{
		Name:  normaliseName(name),
		Title: name,
		Theme: "bear",
	}

	_, ok := s.themes.Lookup(newSite.Theme)
	if !ok {
		return models.Site{}, errors.New("theme not found")
	}

	if err := s.db.InsertSite(ctx, &newSite); err != nil {
		return models.Site{}, err
	}

	return newSite, s.jobs.Queue(ctx, s.sb.CreateNewSite(newSite))
}

func normaliseName(name string) string {
	var sb strings.Builder

	seenDash := false
	for _, r := range strings.TrimSpace(name) {
		if unicode.IsLetter(r) || unicode.IsNumber(r) {
			sb.WriteRune(unicode.ToLower(r))
			seenDash = false
		} else if r == '_' || r == '-' || r == '.' || unicode.IsSpace(r) {
			if !seenDash {
				sb.WriteRune('-')
			}
			seenDash = true
		}
	}
	return sb.String()
}
